#!/usr/bin/env python

"""
Check if a apachedex result matches the desired threshold or raises an error.
"""

import json
import os
import re
import sys
import time
import datetime
import argparse

def checkApachedexResult(apachedex_file, apachedex_report_status_file, desired_threshold):

  if not os.path.isfile(apachedex_file):
    open(apachedex_file, 'a').close()

  with open(apachedex_file, 'r') as content_file:
    content = content_file.read()
 
  if len(content) == 0:
    # File is empty
    # Check the creation date of the file
    # and if the date is greater than 30 hour throw an error
    date_created = os.path.getmtime(apachedex_file)
    current_date = time.mktime(datetime.datetime.now().timetuple())
    if current_date - date_created > 108000:
      with open(apachedex_report_status_file) as f:
        json_content = f.read()

      # Print the message from the monitor report
      if len(json_content) > 0:
        message = json.loads(json_content)["message"]
        return message + "\nFile modification date is greater than 30 hours"
      return "File modification date is greater than 30 hour"
  else:
    #TODO: this is old regex for bash, improve it
    regex = r"Overall<\/h2>.*\n<th>apdex<\/th><th>.*?\n<\/tr><tr>\n<td [^<]*>(.*?)%<\/td>"
    m = re.findall(regex, content)
    if len(m) > 0:
      result=int(m[0])
      if result > desired_threshold:
        return "Thanks for keeping it all clean, result is " + str(result)
      else:
        return "Threshold is lower than expected:  Expected was " + \
               str(desired_threshold) +" and current result is " + str(result)
  return "No result found in the apdex file or the file is corrupted"

def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("--apachedex_file", required=True)
  parser.add_argument("--status_file", required=True)
  parser.add_argument("--threshold", required=True)
  args = parser.parse_args()

  if args.apachedex_file:
    args.apachedex_file = args.apachedex_file + "/ApacheDex-" + datetime.date.today().strftime('%Y-%m-%d') + ".html"

  result = checkApachedexResult(args.apachedex_file, args.status_file, args.threshold)

  print result
  if result != "OK":
    sys.exit(1)



#!/usr/bin/env python

"""
Check if a mariadb result matches the desired threshold or raises an error.
"""

import json
import os
import re
import sys
import time
import datetime
import argparse

def checkMariadbDigestResult(mariadbdex_file, mariadbdex_report_status_file,
                             max_query_threshold, slowest_query_threshold):

  if not os.path.isfile(mariadbdex_file):
    open(mariadbdex_file, 'a').close()

  with open(mariadbdex_file, 'r') as content_file:
    content = content_file.read()
 
  if len(content) == 0:
    # File is empty
    # Check the creation date of the file
    # and if the date is greater than 30 hour throw an error
    date_created = os.path.getmtime(mariadbdex_file)
    current_date = time.mktime(datetime.datetime.now().timetuple())
    if current_date - date_created > 108000:
      with open(mariadbdex_report_status_file) as f:
        json_content = f.read()

      # Print the message from the monitor report
      if len(json_content) > 0:
        message = json.loads(json_content)["message"]
        return message + "\nFile modification date is greater than 30 hours"
      return "File modification date is greater than 30 hour"
  else:
    regex = r"Overall: (.*) total,[\S\s]*# Exec time( *([\d]+)m?s?){4}"
    m = re.findall(regex, content)
    if len(m) > 0:
      total_queries_exec=m[0][0].strip()
      slowest_query_time=int(m[0][2].strip())
      has_k=total_queries_exec[-1:]
      if has_k == "k":
        pre=total_queries_exec[:-1]
        total_queries_exec=float(pre)*1000
      else:
        total_queries_exec=int(total_queries_exec)

      if total_queries_exec < max_query_threshold and slowest_query_time < slowest_query_threshold:
        return "Thanks for keeping it all clean, total queries are : " + str(total_queries_exec) + \
               " slowest query time is : " + str(slowest_query_time)
      else:
        return "Threshold is lower than expected: \nExpected total queries : " + \
               str(max_query_threshold) +" and current is: " + str(total_queries_exec) + "\n"+ \
               "Expected slowest query : " + str(slowest_query_threshold) + " and current is: " + str(slowest_query_time)
  return "No result found in the apdex file or the file is corrupted"

def main():
  parser = argparse.ArgumentParser()
  parser.add_argument("--ptdigest_file", required=True)
  parser.add_argument("--status_file", required=True)
  parser.add_argument("--max_queries_threshold", required=True)
  parser.add_argument("--slowest_query_threshold", required=True)
  args = parser.parse_args()

  result = checkMariadbDigestResult(args.ptdigest_file, args.status_file, args.max_queries_threshold, args.slowest_query_threshold)

  print result
  if result != "OK":
    sys.exit(1)



##############################################################################
#
# Copyright (c) 2017 Vifib SARL and Contributors. All Rights Reserved.
#
# WARNING: This program as such is intended to be used by professional
# programmers who take the whole responsibility of assessing all potential
# consequences resulting from its eventual inadequacies and bugs
# End users who are looking for a ready-to-use solution with commercial
# guarantees and support are strongly adviced to contract a Free Software
# Service Company
#
# This program is Free Software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
##############################################################################

import unittest
import os
import time
import tempfile

from slapos.test.promise import data
from slapos.promise.check_slow_queries_digest_result import checkMariadbDigestResult

class TestCheckSlowQueriesDigestResult(unittest.TestCase):

  def setUp(self):

    _, self.empty_ptdigest_file = tempfile.mkstemp()
    _, self.old_ptdigest_file = tempfile.mkstemp()
    _, self.status_file = tempfile.mkstemp()

    self.base_path = "/".join(data.__file__.split("/")[:-1])

  def test_pass(self):
    self.assertEquals("Thanks for keeping it all clean, total queries are : 3420.0 slowest query time is : 34",
      checkMariadbDigestResult(self.base_path + "/ptdigest.html", self.status_file, 5000, 100))

  def test_empty_file(self):
    self.assertEquals("No result found in the apdex file or the file is corrupted",
      checkMariadbDigestResult(self.empty_ptdigest_file, self.status_file, 60, 100))

  def test_fail(self):
    self.assertEquals("Threshold is lower than expected: \nExpected total queries : 90 and current is: 3420.0\nExpected slowest query : 100 and current is: 34",
      checkMariadbDigestResult(self.base_path + "/ptdigest.html", self.status_file, 90, 100))

  def test_old_file(self):
    os.utime(self.old_ptdigest_file, (time.time() - 202800, time.time() - 202800))
    self.assertEquals("File modification date is greater than 30 hour",
      checkMariadbDigestResult(self.old_ptdigest_file, self.status_file, 60, 100))

if __name__ == '__main__':
  unittest.main()

